C     PROGRAMA PER SIMULAR EL MODEL D'ISING 3-D                   *
C      Metodes de Simulacio Numerica.                             *
C      Marc Pradas. 8 de Desembre de 2004                         *
C******************************************************************
C     Parametres inicials                                         * 
C     llav = llavor inicial per generar els nombre aleatoris      *
C     j = valor de la interaccio                                  *
C     ta = temperatura inicial a estudiar                          *
C     tb = temperatura final                                      *
C     mcs = nombre de vegades que fa l'algorisme de metropolis    * 
C     iconf = tipus de configuracio inicial                       * 
C            1 = ferromagnetica                                   *
C	     2 = aleatoria                                        *
C	     3 = antiferromagnetica                               *
C******************************************************************
      
      PROGRAM is3d	      
      Implicit real*8(a-h,j,t)
      Character*12 lpfile
      Character*2 lhp(0:31),nh
      Parameter(ls=21)
      Parameter(lv=(ls-1)**3)
      Dimension s(ls,ls,ls)
      Dimension pbc(0:ls),camp(0:31)

      Data camp/0.0,0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0,1.1,1.2,
     +1.3,1.4,1.5,1.6,1.7,1.8,1.9,2.0,2.1,2.2,2.3,2.4,2.5,2.6,2.7,
     +2.8,2.9,3.0,3.1/
      Data lhp/'00','01','02','03','04','05','06','07','08','09',
     +'10','11','12','13','14','15','16','17','18','19','20','21',
     +'22','23','24','25','26','27','28','29','30','31'/
      
C     Parametres de la simulacio

      llav=9998
      j=-1.0
      ta=0.4
      tb=10.0
      mcs=13000
      iconf=3

C     Condicions periodiques de contorn

      pbc(0)=ls-1
      pbc(ls)=1

      Do i=1,ls-1
       pbc(i)=i
      End do
      
C     Fa simulacions per diferents valors del camp
      
      Do ih=0,20
       h=camp(ih)
       nh=lhp(ih)
       lpfile='isH'//nh//'L20.dat'
       Open(11,file=lpfile)       
       write(*,*)'Camp aplicat = ',h

C      Configuracio inicial 1: estat ferromagnetic

       If(iconf.eq.1) then
        Do ii=1,ls-1
         Do jj=1,ls-1
          Do kk=1,ls-1
           s(ii,jj,kk)=1
          end do
         end do
        end do
        go to 1122
       End if

C      Configuracio inicial 2: aleatoria

       If(iconf.eq.2) then
        nu=0
        nd=0
        Do ii=1,ls-1
         Do jj=1,ls-1
          Do kk=1,ls-1
 1111      ax=ran3(llav)
           ni=int(ax*2)
           if(ni.eq.1) then
            s(ii,jj,kk)=1
            nu=nu+1
           end if
           if(ni.eq.0) then
            s(ii,jj,kk)=-1
            nd=nd+1
           end if
           if(ni.eq.2) go to 1111
          End do
         End do
        End do
        go to 1122
       End if

C      Configuracio inicial 3: estat antiferromagnetic       

       If(iconf.eq.3) then
        Do i=1,ls-1
         Do k=1,ls-1
          Do l=1,ls-1
           if(mod(i+k+l,2).ne.0) then
            s(i,k,l)=1
           else
            s(i,k,l)=-1
           end if
          end do
         end do
        end do
        go to 1122
       End if
 1122  continue
   
C      Fem un bucle per totes les temperatures  
          
       t=ta
       
       Do while(t.le.tb)
	if((t.gt.3.8).and.(t.lt.4.7)) then
	 nmcs=80000
         ipromig=40000
	 dt=0.01
	else
	 dt=0.1
	 nmcs=50000
	 ipromig=10000
	end if
	t=t+dt
	
C      Calcula l'energia i el parametre d'ordre inicial
      
       energia=0.0d0

       Do i=1,ls-1
        Do k=1,ls-1      
         Do l=1,ls-1
          energia = energia +s(i,k,l)*(j*(s(pbc(i+1),k,l)
     1    +s(i,pbc(k+1),l)+s(i,k,pbc(l+1)))+h)
	 End do
        End do
       End do
       energia=-energia

C      Les dues subxarxes s'escullen de tal forma que si la suma dels
C      tres indexs es senar estem a la subxarxa A, i si es parell
C      estem a la B.
C      Per calcular el parametre d'ordre, fa la suma dels espins a
C      parelles d'una mateixa subxarxa.

       poa=0
       pob=0

c       open(44,file='xarxaa.dat',status='unknown')
c       open(55,file='xarxab.dat',status='unknown')

       Do l=1,ls-1
        If(mod(l,2).ne.0)then
         Do ia=1,ls-1,2
          ia2=ia+1
          Do k=1,ls-1,2
           k2=k+1
           poa=poa+s(ia,k,l)+s(ia2,k2,l)
c           write(44,*)ia,k,l
c           write(44,*)ia2,k2,l
          End do
         End do
         Do ib=2,ls-1,2
          ib2=ib-1
          Do k=1,ls-1,2
           k2=k+1
           pob=pob+s(ib,k,l)+s(ib2,k2,l)
c           write(55,*)ib,k,l
c           write(55,*)ib2,k2,l           
          End do
         End do
        Else
         Do ia=2,ls-1,2
          ia2=ia-1
          Do k=1,ls-1,2
           k2=k+1
           poa=poa+s(ia,k,l)+s(ia2,k2,l)
c           write(44,*)ia,k,l
c           write(44,*)ia2,k2,l  
          End do
         End do
         Do ib=1,ls-1,2
          ib2=ib+1
          Do k=1,ls-1,2
           k2=k+1
           pob=pob+s(ib,k,l)+s(ib2,k2,l)
c           write(55,*)ib,k,l
c           write(55,*)ib2,k2,l          
          End do
         End do
        End if
       End do

       write(*,*) lv,poa,pob
c       close(44)
c       close(55)

c       open(22,file='fluctpo.dat',status='unknown')

C      Algorisme de Metropolis, spin-flip.
      
       energs=0.0
       energs2=0.0
       dpos=0.0
       pos2=0.0
       poanti=0.0
       poferro=0.0
       Do inmcs=1,nmcs
        nlt=0
        ngt=0
        Do imcs=1,mcs
         x1=ran3(idum)
         i=1+int((ls-1)*x1)
         x2=ran3(15*i)
         k=1+int((ls-1)*x2)
         x3=ran3(k*i)
         l=1+int((ls-1)*x3)

         asumv=s(pbc(i+1),k,l)+s(pbc(i-1),k,l)+s(i,pbc(k+1),l)+
     1   s(i,pbc(k-1),l)+s(i,k,pbc(l+1))+s(i,k,pbc(l-1))

         de=(2.0*j*s(i,k,l)+h)*asumv
         if(de.lt.0) then
          s(i,k,l)=-s(i,k,l)
          energia=energia+de
          if(mod(i+k+l,2).ne.0) then
           poa=poa+2*s(i,k,l)
          else
           pob=pob+2*s(i,k,l)
          end if
          nlt=nlt+1
         end if
         if(de.gt.0) then
          dex=exp(-de/t)
          ngt=ngt+1
          y=ran3(int(dex**2))
          if(y.lt.dex) then
	   s(i,k,l)=-s(i,k,l)
           energia=energia+de
           if(mod(i+k+l,2).ne.0) then
            poa=poa+2*s(i,k,l)
           else
            pob=pob+2*s(i,k,l)
           end if
          end if
         end if
        End do       
c         write(*,*)nlt,ngt
	if(inmcs.gt.ipromig) then
         energs=energs+energia/dfloat(lv)
         energs2=energs2+(energia/dfloat(lv))**2
         poanti=poanti+abs(poa-pob)/dfloat(lv)
	 poferro=poferro+abs(poa+pob)/dfloat(lv)
         dpos=dpos+abs((poa+pob)/dfloat(lv))
         pos2=pos2+((pob+poa)/dfloat(lv))**2 
	end if
	
c        write(22,*)inmcs,abs(poa-pob)/dfloat(lv)
       End do

C      Calculem els promitjos d'interes

       ipnmcs=nmcs-ipromig

       em=energs/dfloat(ipnmcs)
       e2m=energs2/dfloat(ipnmcs)
       vare=e2m-em**2
       cv=lv*(vare)/(t**2)
       dpom=dpos/dfloat(ipnmcs)
       po2m=pos2/dfloat(ipnmcs)
       poferro=poferro/dfloat(ipnmcs)
       poanti=poanti/dfloat(ipnmcs)
       susc=lv*(po2m-dpom**2)/(t**2)
       
       write(11,555)t,em,e2m,poferro,poanti,cv,susc
 555   format(f5.2,2x,f16.5,2x,f16.3,2x,f7.4,2x,f7.4,2x,f22.5,2x,f12.9)   
      
       write(*,555)t,em,e2m,poferro,poanti,cv,susc

       End do

       Close(11) 
c      Close(22)
      End do
 1112 END